angular.module('fw.controllers', [])

    .controller('TabsController', function($scope, $state) {
        //kind of a hack to get the presenters to go to top of page
        //otherwise if go to presenters from session detail link
        //and then go to map and then back to presenters tab via tab button
        //would end up back on the prior presenter anchor 
        //this forces back to the top since anchor value soe not exist
        $scope.goToPresentersTop = function() {
            $state.go('tab.presenters', {id: -1});
        }
    })

    .controller('SessionsCtrl', function ($scope, $rootScope, $log, SessionsService, $state, $ionicPopover, $ionicPopup, $window, $cordovaSocialSharing, $ionicLoading, $ionicScrollDelegate, $timeout) {
        // With the new view caching in Ionic, Controllers are only called
        // when they are recreated or on app start, instead of every page change.
        // To listen for when this page is active (for example, to refresh data),
        // listen for the $ionicView.enter event:
        //
        //$scope.$on('$ionicView.enter', function(e) {
        //});

        //$scope.sessions = SessionsService.all();

        var starFilterLocked = false;
        
        $scope.stars = [];
        $scope.starFilterOn = false;
        $scope.shownGroups = [];

        init();

        function init() {
            $rootScope.loadingMessage = 'Loading sessions...';
            $ionicLoading.show()
                .then(function() {
                    return SessionsService.all();
                })
                .then(function (sessions) {
                    //$scope.sessions = sessions;
                    $scope.groupedSessions = sessions;
                    loadStars();
                })
                .finally(function() {
                    $ionicLoading.hide();
                });
        }

        $scope.toggleStarFilter = function(evt) {

            //hacky fix for iOS devices to get around ng-click getting fired 2 times
            //if (!starFilterLocked) {
            //    starFilterLocked = true;

                //fix for iOS only
                evt.preventDefault();
                evt.stopPropagation();
            
                $log.debug('toggleStarFilter called: ' + JSON.stringify(evt));
                $scope.starFilterOn = !$scope.starFilterOn;
                
                if ($scope.starFilterOn) {
                    $ionicScrollDelegate.$getByHandle('mainScroll').scrollTop();
                }

                $timeout(function() {
                    $ionicScrollDelegate.$getByHandle('mainScroll').resize();
                }, 100);

                //$timeout(function() {
                //    starFilterLocked = false;
                //}, 200);
            //}
        }

        $scope.starredFilter = function(value, index, array) {
            if ($scope.starFilterOn) {
                return $scope.stars.indexOf(value.id) > -1;
            } else {
                return true;
            }
        }

        $scope.toggleGroup = function(group) {
            if ($scope.isGroupShown(group)) {
                $window._.pull($scope.shownGroups, group.displayDate);
            } else {
                $scope.shownGroups.push(group.displayDate);
            }

            $timeout(function() {
                $ionicScrollDelegate.resize();
            }, 100);
        };

        $scope.isGroupShown = function(group) {
            return $scope.shownGroups.indexOf(group.displayDate) > -1;
        };

        $scope.forceRefresh = function() {
            SessionsService.all()
                .then(function (sessions) {
                    $scope.groupedSessions = sessions;
                    loadStars();
                })
                .finally(function() {
                    // Stop the ion-refresher from spinning
                    $scope.$broadcast('scroll.refreshComplete');
                    $ionicLoading.hide();
                });
        }

        function loadStars() {
            var starsStr = localStorage.getItem('stars');
            if (starsStr) {
                $scope.stars = JSON.parse(starsStr);
            }
        }

        $scope.isStarred = function(sessionId) {
            var includes = $window._.includes($scope.stars, sessionId);
            return includes; 
        }

        $scope.toggleStar = function(evt, id) {
            evt.preventDefault();
            evt.stopPropagation();

            $log.debug('toggle star ' + id);
            if ($scope.isStarred(id)) {
                //unstar
                $window._.pull($scope.stars, id);
            } else {
                //star
                $scope.stars.push(id);
            }

            localStorage.setItem('stars', JSON.stringify($scope.stars));
        }

        $scope.showSessionDetails = function(showDetails, id) {
            if (showDetails) {
                $state.go('tab.sessions-detail', {sessionId: id});
            }
            //href="#/tab/sessions/{{session.id}}
        }

        $scope.showMenuPopup = function($event) {
            $ionicPopover.fromTemplateUrl('templates/sessionsPopover.html', {
                scope: $scope
            }).then(function(popover) {
                $scope.popover = popover;
                $scope.popover.show($event);
            });

            //Cleanup the popover
            $scope.$on('$destroy', function() {
                $scope.popover.remove();
            });
        }

        $scope.showAbout = function() {
            $scope.popover.hide();
            $ionicPopup.alert({
                templateUrl: 'templates/about.html'
            })
        }

        $scope.shareAllNotes = function(evt) {
            //fix for iOS only
            evt.preventDefault();
            evt.stopPropagation();

            var lsKeys = Object.keys(localStorage);

            var noteKeys = $window._.filter(lsKeys, function(key) {
                return key.indexOf('_note') > -1;
            });

            var emailBody = '';
            var anyNotes = false;
            $window._.forEach(noteKeys, function(key) {
                var sessionId = key.split('_')[0];
                $window._.forEach($scope.groupedSessions, function(group) {
                    var session = $window._.find(group.sessions, {id: sessionId});
                    var note = localStorage.getItem(key);
                    if (session && note && note.length) {
                        anyNotes = true;
                        emailBody = emailBody + session.title + ': \n' + note + '\n\n';
                    }
                });
            });

            if (!anyNotes) {
                //show alert
                $ionicPopup.alert({
                    template: 'There are currrently no notes to share.'
                });
                return;
            }

            $log.debug(emailBody);
            $scope.popover.hide();
            if ($window.cordova) {
                $cordovaSocialSharing
                    .share(emailBody, 'Faithwalkers notes', null, null) // Share via native share sheet
                    .then(function(result) {
                        $log.debug('share success: ' + result);
                    }, function(err) {
                        $ionicPopup.alert({
                            title: '',
                            template: 'Unable to share notes: ' + err
                        });
                    });
            } else {
                $log.log('not on a device -- unable to share notes');
            }
            
        }

    })

    .controller('SessionDetailCtrl', function ($scope, $rootScope, $stateParams, $ionicHistory, $log, SessionsService, $window, $cordovaSocialSharing, $ionicPopup) {
        $scope.session = SessionsService.get($stateParams.sessionId);
        $scope.data = {
            note: ''
        };

        $scope.$on('$ionicView.beforeEnter', function() {
            //load note from localStorage
            var note = localStorage.getItem($scope.session.id+"_note");
            if (note) {
                $scope.data.note = localStorage.getItem($scope.session.id+"_note");
            }
        });

        $scope.$on('$ionicView.afterEnter', resizeNotesTextArea);

        $scope.$on('$ionicView.beforeLeave', function() {
            //save notes when leaving session detail view
            localStorage.setItem($scope.session.id+"_note", $scope.data.note);
        });

        function resizeNotesTextArea() {
            if (cordova && cordova.platformId === 'ios') {
                var screenHeight = window.screen.height * window.devicePixelRatio;
                if (screenHeight <= 1136) {
                    var element = document.getElementById('notesTextArea');
                    if (element) {
                        element.style.height = '200px';
                    }
                }
            }
            /*
            var element = document.getElementById('notesTextArea');
            if (element) {
                $log.debug('resizing text area: ' + element.scrollHeight);
                if (parseInt(element.scrollHeight) > 250) {
                    element.style.height = '250px';
                } else {
	                element.style.height =  element.scrollHeight + 'px';
                }
            }*/
        }

        $scope.goToTeachLink = function(link) {
            if ($window.cordova) {
                cordova.InAppBrowser.open(link, '_blank', 'location=yes,zoom=noteKeys');
            } else {
                $log.debug('cannot load link -- not on a device');
            }
        }

        $scope.shareSessionNotes = function() {
            if (!$scope.data.note || !$scope.data.note.length) {
                $ionicPopup.alert({
                    template: 'There is no note content to share.'
                });
                return;
            }

            var emailBody = $scope.session.title + '\n' + $scope.data.note;
            $log.debug(emailBody);
            if ($window.cordova) {
                $cordovaSocialSharing
                    .share(emailBody, $scope.session.title + 'Notes', null, null) // Share via native share sheet
                    .then(function(result) {
                        $log.debug('share success: ' + result);
                    }, function(err) {
                        $ionicPopup.alert({
                            title: '',
                            template: 'Unable to share notes: ' + err
                        });
                    });
            } else {
                $log.log('not on a device -- unable to share session detail note');
            }
        }

        $rootScope.$ionicGoBack = function(backCount) {
            $ionicHistory.goBack(backCount);
        };
    })

    .controller('PresentersCtrl', function ($scope, $log, SessionsService, $location, $ionicScrollDelegate, $state, $stateParams, $timeout) {

        $scope.$on('$ionicView.beforeEnter', function() {
            SessionsService.getPresenters()
                .then(function(presenters) {
                    $scope.presenters = presenters;
                });
        });

        $scope.$on('$ionicView.enter', function(evt, data) {
            scrollToPresenter(data.stateParams.id);
        });

        function scrollToPresenter(id) {
            $timeout(function() {
                    $location.hash(id);
                    $ionicScrollDelegate.$getByHandle('mainScroll').anchorScroll(true);
            }, 0);
        }

        
    })

    .controller('MapCtrl', function ($scope, $log, $window) {

    });
