angular.module('fw.services', [])

    .factory('SessionsService', function ($q, $http, $log, $window) {

        var data = null,
            transformedSessions = null,
            groupedSessions = null;

        return {
            all: function () {
                var deferred = $q.defer();
                //http://www.beggarsgate.com/files/theme/data/faithwalkers.json

                //https://dl.dropboxusercontent.com/s/8hs5eix664otv00/faithwalkers.json?dl=1
                //https://app.box.com/s/f26mc7q7cmhwdkaw56meeo31gkhtf1ys
                $http.get('https://dl.dropboxusercontent.com/s/8hs5eix664otv00/faithwalkers.json?dl=1')
                    .then(function (response) {
                        $log.debug(response.data);
                        localStorage.setItem('faithwalkers.json', JSON.stringify(response.data));
                        data = response.data;

                        transformSessions(data.sessions)
                        //deferred.resolve(transformedSessions);
                        deferred.resolve(groupedSessions);

                    })
                    .catch(function() {
                        //try to load from localStorage if there is a problem downloading
                        var d = localStorage.getItem('faithwalkers.json');
                        if (d) {
                            data = JSON.parse(d);
                            transformSessions(data.sessions)
                            deferred.resolve(groupedSessions);
                        } else {
                            deferred.reject('Unable to load session data.');
                        }
                    });

                return deferred.promise;
            },

            get: function (sessionId) {
                var session = null;
                for (var i = 0; i < transformedSessions.length; i++) {
                    if (transformedSessions[i].id === sessionId) {
                        session = transformedSessions[i];
                    }
                }

                return session;
            },

            getPresenters: function () {
                var deferred = $q.defer();

                if (data) {
                    var p = $window._.orderBy(data.presenters, ['id'], ['asc']);
                    deferred.resolve(p);
                } else {
                    all().then(function () {
                        var p = $window._.orderBy(data.presenters, ['id'], ['asc']);
                        deferred.resolve(p);
                    });
                }

                return deferred.promise;
            }
        }

        /**
         * Adds session presenters to all sessions.
         */
        function transformSessions(sessions) {
            angular.forEach(sessions, function (session) {
                session.presenters = [];
                angular.forEach(session.presenterIds, function (presenterId) {
                    angular.forEach(data.presenters, function (presenter) {
                        if (presenter.id === presenterId) {
                            session.presenters.push(presenter);
                        }
                    });
                });

                //add datetime fields for sorting and display with day
                //"date": "12/30/2016",
                //"startTime": "1:15 pm",
                var m = moment(session.date + ' ' + session.startTime, 'MM/DD/YYYY h:mm a');
                session.dateTimeISO = m.format();
                session.displayDateTime = m.format('dddd, MM/DD/YYYY');
            });

            transformedSessions = sessions;
            transformedSessions = $window._.orderBy(transformedSessions, ['dateTimeISO', 'title'], ['asc', 'asc']);
            groupSessionsByDisplayDate(transformedSessions);
            
            return transformSessions;
        }

        function groupSessionsByDisplayDate(sessions) {
            var uniqByDate = $window._.uniqBy(sessions, 'displayDateTime');
            var dateGroups = $window._.map(uniqByDate, function(session) {
                return {displayDate: session.displayDateTime, sessions: []};
            });

            $window._.forEach(dateGroups, function(group) {
                var fSessions = $window._.filter(transformedSessions, {displayDateTime: group.displayDate});
                group.sessions = fSessions;
            });

            groupedSessions = dateGroups;
        }




    });
